library(lavaan)

##################################################
##################################################
# 2smm exercise 2 (optional)
##################################################
##################################################

setwd("C:/holger/Workshops/nonlinear/final/sem-3/data")
dat <- read.table("pisa_online1.dat",header=T,na.strings = "999")

##################################################
# in the PISA data set 2009 (OECD, 2012)
##################################################
dat <- read.table("../data/pisa_online1.dat",header=T,na.strings = "999")
#delete missings: nlsem does not allow missing at the moment
dat <- na.omit(dat)

dim(dat)

# sample size
n <- dim(dat)[1]

# sorting of dataframe
dat2 <- data.frame(dat$read2,dat$read3,
                   dat$on2,dat$on3,
                   dat$att2,dat$att3,
                   dat$read1,dat$on1,dat$att1)

colnames(dat2) <- c("y2","y3","x2","x3","x5","x6","y1","x1","x4")

########################################################
# stage 1: factor scores and cfa
########################################################

model <- '
y2 ~ 1
y3 ~ 1
x2 ~ 1
x3 ~ 1
x5 ~ 1
x6 ~ 1
eta=~ y1 + y2 + y3
xi1=~ x1 + x2 + x3
xi2=~ x4 + x5 + x6
y2 ~~y2
y3 ~~y3
x2 ~~x2
x3 ~~x3
x5 ~~x5
x6 ~~x6
y1 ~~y1
x1 ~~x1
x4 ~~x4
xi1~~ xi2
xi1~~ eta
xi2~~ eta
y1 ~ 0*1
x1 ~ 0*1
x4 ~ 0*1
xi1 ~ 1
xi2 ~ 1
eta ~ 1
'

cfa <- sem(model, data=dat2, estimator="MLR")
summary(cfa)

################
# Extract parameter estimates
################
tx  <- coef(cfa)[1:6]
lx  <- coef(cfa)[7:12]
eps <- coef(cfa)[13:21]

################
# Factor scores
################
# residual variances
psihat <-diag(9)*eps

# factor loadings
beta1hat <- matrix(0,6,3)
beta1hat[1:2,1] <- lx[1:2]
beta1hat[3:4,2] <- lx[3:4]
beta1hat[5:6,3] <- lx[5:6]

# intercepts
beta0hat <- matrix(tx,6,1)

# 
b1.mod1 <- cbind(matrix(c(0),3,6),diag(3))
b1.mod2 <- rbind(diag(6),-1*t(beta1hat))

# eq(6)
Gamma <- b1.mod1 %*% psihat %*% b1.mod2 %*% solve(t(b1.mod2) %*% psihat %*% (b1.mod2))

# eq(5): factor scores
fs<-matrix(c(0),n,3)
for(i in 1:n){
  fs[i,] <- cbind(-Gamma,(diag(3)+ Gamma %*% beta1hat))%*%(t(as.matrix(dat2[i,]))-rbind(beta0hat,matrix(0,3,1)))
}

f1 <- fs[,1] # eta, read
f2 <- fs[,2] # online, xi1
f3 <- fs[,3] # attitude, xi2

################
# unreliability of factor score estimates
################
# 1 (iii)
# eq (7)
sigmaee <- cbind(-Gamma,(diag(3)+ Gamma %*% beta1hat)) %*% psihat %*%  t(b1.mod1)


########################################################
# stage 2: structural model
########################################################
# Assume simple models (Scenario 3 in W&A2003)
# mu 3rd moments & nu 4th moments
# 1 (iv)
mu123 <-0  #e1*e2*e3   mu13
mu122 <-0  #e1*e2^2
mu133 <-0  #e1*e3^2
mu223 <-0  #e2^2*e3    mu23
mu233 <-0  #e2*e3^2    mu33
mu222 <-0  #e2^3
mu333 <-0  #e3^3

nu2233<-sigmaee[2,2]*sigmaee[3,3] #e2^2*e3^2 mu14
nu2223<-sigmaee[2,2]*sigmaee[2,3] #e2^3*e3
nu2333<-sigmaee[2,3]*sigmaee[3,3] #e2*e3^3
nu2222<-sigmaee[2,2]*sigmaee[2,2] #e2^4
nu3333<-sigmaee[3,3]*sigmaee[3,3] #e3^4

# 2 (ii)
# elements of M^
ai11<-rep(1,n)
ai21<-f2
ai31<-f3
ai41<-f2*f3-sigmaee[2,3] 
ai51<-f2^2-sigmaee[2,2] #
ai61<-f3^2-sigmaee[3,3] #
ai22<-ai51
ai32<-f2*f3-sigmaee[2,3]
ai42<-f2^2*f3-2*sigmaee[2,3]*f2-sigmaee[2,2]*f3-mu223
ai52<-f2^3-3*f2*sigmaee[2,2]-mu222   #
ai62<-f2*f3^2-2*sigmaee[2,3]*f3-sigmaee[3,3]*f2-mu233   #
ai33<-ai61
ai43<-ai62
ai53<-ai42   #
ai63<-f3^3-3*f3*sigmaee[3,3]-mu333   #
ai44<-f2^2*f3^2-sigmaee[3,3]*f2^2-sigmaee[2,2]*f3^2-4*sigmaee[2,3]*f2*f3+2*sigmaee[2,2]*sigmaee[3,3]+4*sigmaee[2,3]^2-2*mu233*f2-2*mu223*f3-nu2233
ai54<-f2^3*f3-3*sigmaee[2,3]*f2^2-3*sigmaee[2,2]*f2*f3+6*sigmaee[2,2]*sigmaee[2,3]-3*mu223*f2-mu222*f3-nu2223  #
ai64<-f2*f3^3-3*sigmaee[2,3]*f3^2-3*sigmaee[3,3]*f2*f3+6*sigmaee[2,3]*sigmaee[3,3]-3*mu233*f3-mu333*f2-nu2333  #
ai55<-f2^4-6*sigmaee[2,2]*f2^2+6*sigmaee[2,2]^2-4*mu222*f2-nu2222   #
ai65<-ai44
ai66<-f3^4-6*sigmaee[3,3]*f3^2+6*sigmaee[3,3]^2-4*mu333*f3-nu3333   #

# elements of m^
di1<-f1
di2<-f1*f2-sigmaee[1,2]
di3<-f1*f3-sigmaee[1,3]
di4<-f1*f2*f3-sigmaee[1,2]*f3-sigmaee[1,3]*f2-sigmaee[2,3]*f1-mu123
di5<-f1*f2^2-sigmaee[2,2]*f1-2*sigmaee[1,2]*f2-mu122    #
di6<-f1*f3^2-sigmaee[3,3]*f1-2*sigmaee[1,3]*f3-mu133    #

Mhat<-matrix(1/n*c(sum(ai11),sum(ai21),sum(ai31),sum(ai41),sum(ai51),sum(ai61),
                   sum(ai21),sum(ai22),sum(ai32),sum(ai42),sum(ai52),sum(ai62),
                   sum(ai31),sum(ai32),sum(ai33),sum(ai43),sum(ai53),sum(ai63),
                   sum(ai41),sum(ai42),sum(ai43),sum(ai44),sum(ai54),sum(ai64),
                   sum(ai51),sum(ai52),sum(ai53),sum(ai54),sum(ai55),sum(ai65),
                   sum(ai61),sum(ai62),sum(ai63),sum(ai64),sum(ai65),sum(ai66)),6,6)

mhat<-matrix(1/n*c(sum(di1),sum(di2),sum(di3),sum(di4),sum(di5),sum(di6)),6,1)

# alpha^ = c(alpha1,gamma1,gamma2,omega12,omega11,omega22)
# 2 (iii)
# these are the regression coefficients
alpha <-solve(Mhat)%*%mhat

##########################################################################################
##########################################################################################
##########################################################################################

#SE estimation
# NOTE: these are uncorrected SE (W&A, 2003, use a more complicated approach)

omegastar<-matrix(0,6,6)

for(i in 1:n){
  Mihat<- matrix(c(ai11[i],ai21[i],ai31[i],ai41[i],ai51[i],ai61[i],
                   ai21[i],ai22[i],ai32[i],ai42[i],ai52[i],ai62[i],
                   ai31[i],ai32[i],ai33[i],ai43[i],ai53[i],ai63[i],
                   ai41[i],ai42[i],ai43[i],ai44[i],ai54[i],ai64[i],
                   ai51[i],ai52[i],ai53[i],ai54[i],ai55[i],ai65[i],
                   ai61[i],ai62[i],ai63[i],ai64[i],ai65[i],ai66[i]),6,6)
  
  mihat<- matrix(c(di1[i],di2[i],di3[i],di4[i],di5[i],di6[i]),6,1)
  la   <- mihat-Mihat%*%alpha
  omegastar.i <- la%*%t(la)
  omegastar  <-omegastar+omegastar.i
}

omegahat <- 1/n*omegastar 

#eq (12)
Vhat <- 1/n * solve(Mhat)%*% omegahat %*% solve(Mhat)
se <- sqrt(diag(Vhat)) 

# FINAL RESULTS
results <- cbind(alpha,se,alpha/se,1-pnorm(abs(alpha/se)))
colnames(results) <- c("estimate","se","t","p")

round(results,3)






